@@ -1,7 +1,7 @@
-// bs3 popover and tooltips on button with hover
 $(document).ready(function(){
     $('[pop-toggle="hover"]').popover({
-        trigger : 'hover'
+        trigger : 'hover',
+        container: 'body'
     });
     $('[data-toggle="tooltip"]').tooltip();
 });
@@ -9,6 +9,7 @@ $(document).ready(function(){
 // popover to work on dynamically added elements
 $(document).on("DOMNodeInserted", '[pop-toggle="hover"]', function(){
     $(this).popover({
-        trigger : 'hover'
+        trigger : 'hover',
+        container: 'body'
     });
 });
